<?php
/*
Plugin Name: mimeTeX
Plugin URI: http://capivara.free.fr/tony
Description: Use &#091;tex&#093;&#091;/tex&#093; tags to embed LaTeX math in posts, see the <a href="http://www.forkosh.dreamhost.com">mimeTeX manual</a> for details. The original Version 1.1 has been tweaked many times!
Version: 1.1.3
Author: Anders Dahnielson
Author URI: http://en.dahnielson.com/
*/

/*  
    Copyright 2006  Anders Dahnielson (email : anders@dahnielson.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
    Version-History:
    1.1.3:
    Tweaker: Tony Lopes
    Email:   tony.lopes@gmail.com
    Homepage: http://capivara.free.fr/tony
    -   Pointing to a more static location to get the images (I promise!)
    -   Some error handling when images are not generated
    -   Removing html tags and entities

    1.1.2:
    Tweaker: Martin Becker
    Email:   mbecker@physik.uni-wuerzburg.de
    Homepage: http://fstyle.de
    -   Edited discription of plugin, so correct tags will be shown
    -   Put inclusion of Snoopy-Class down into the class (mimetex), 
        for in some cases WP will intialize the Snoopy-Class of its own
        after we do here, causing it to not to be able to reinitialize.
        For some reason I don't understand. 
        Note: I think all this workaround using our own extra Snoopy-Class can be
        fixed by the wordpress-team by removing the if statement in their Snoopy-Class.
        That would be the case for ./wp-includes/rss-functions.php in ./wp-admin/index.php
    -   Cleaner code
    
    1.1.1:
    Tweaker: Baris Evrim Demiroz
    Email:   b.evrim /AT/ anlak /DOT/ com
    Homepage: http://www.anlak.com
    -   Switched to Snoopy
    -   Working in comments as well: Thanks to Robert Jones, he gave the code snippet for plugin to work with comments. You may wish to visit him: http://www.jonesieboy.co.uk/blog
    
    1.1.0:
    Anders Dahnielson, URI: http://dahnielson.com
    -   Anders Dahnielson's original version.
*/

function html_entity_decode_utf8($string)
{
    static $trans_tbl;
   
    // replace numeric entities
    $string = preg_replace('~&#x([0-9a-f]+);~ei', 'code2utf(hexdec("\\1"))', $string);
    $string = preg_replace('~&#([0-9]+);~e', 'code2utf(\\1)', $string);

    // replace literal entities
    if (!isset($trans_tbl))
    {
        $trans_tbl = array();
       
        foreach (get_html_translation_table(HTML_ENTITIES) as $val=>$key)
            $trans_tbl[$key] = utf8_encode($val);
    }
   
    return strtr($string, $trans_tbl);
}

// Returns the utf string corresponding to the unicode value (from php.net, courtesy - romans@void.lv)
function code2utf($num)
{
    if ($num < 128) return chr($num);
    if ($num < 2048) return chr(($num >> 6) + 192) . chr(($num & 63) + 128);
    if ($num < 65536) return chr(($num >> 12) + 224) . chr((($num >> 6) & 63) + 128) . chr(($num & 63) + 128);
    if ($num < 2097152) return chr(($num >> 18) + 240) . chr((($num >> 12) & 63) + 128) . chr((($num >> 6) & 63) + 128) . chr(($num & 63) + 128);
    return '';
}

class mimetex {
    // parsing the text to display tex by putting tex-images-tags into the code created by createTex
    function parseTex ($toParse) {
        // tag specification (which tags are to be replaced)
        $regex = '#\[tex\](.*?)\[/tex\]#si';
	return preg_replace_callback($regex, array(&$this, 'createTex'), $toParse);
    }
    
    // reading the tex-expression and create an image and a image-tag representing that expression
    function createTex($toTex) {	
	
    	$formula_text = html_entity_decode_utf8( strip_tags($toTex[1]));	
	$formula_hash = md5($formula_text);
	$formula_filename = 'tex_'.$formula_hash.'.gif';

	$cache_path = ABSPATH . '/wp-content/cache/';
	$cache_formula_path = $cache_path . $formula_filename;
	$cache_url = get_bloginfo('wpurl') . '/wp-content/cache/';
	$cache_formula_url = $cache_url . $formula_filename;
    
        if (!is_file($cache_formula_path)) {
     		if (!class_exists('Snoopy')) require_once ('Snoopy.class.php');
		$snoopy = new Snoopy;
		// Fetching image and testing if it's correct
		if($snoopy->fetch('http://capivara.free.fr/mimetex/mimetexgenerator.php?code='.rawurlencode($formula_text)) && substr($snoopy->results, 0, 3) == "GIF"){
			// this will copy the created tex-image to your cache-folder	
			$cache_file = fopen($cache_formula_path, 'w');
            		fputs($cache_file, $snoopy->results);
			fclose($cache_file);
			// returning the image-tag, referring to the image in your cache folder 
        		
		}else{
			return "Mimetex Image for <b>$formula_text</b> could not be generated ";
		}
        }
	return "<img src=\"$cache_formula_url\" class=\"tex\" alt=\"$formula_text\" />";
    }
}

$mimetex_object = new mimetex;
// this specifies where parsing should be done. one can look up further information on wordpress.org
add_filter('the_title', array($mimetex_object, 'parseTex'));
add_filter('the_content', array($mimetex_object, 'parseTex'));
add_filter('the_excerpt', array($mimetex_object, 'parseTex'));
add_filter('comment_text', array($mimetex_object, 'parseTex'));

?>
